/* pnmposter VERSION 0.1
 * Copyright 1998 Christoph Stahl (stahl@studcs.uni-sb.de)
 * homepage: http://fsinfo.cs.uni-sb.de/~stahl/pnmposter/
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define INCH 0.03937007874 /* 1 mm */

FILE *fp;
char filename[256];/* filename */
char casper[256]; /* linebuffer */
char tmpname[64]; /* tmpfile name */
char outname[256];/* output filename */

int imagex, imagey; 	/* ppm-image size		*/
double scale;		/* scalefactor (72/dpi)		*/
int dpi;		/* 300 dpi			*/
int devpix;		/* as in pnmtops		*/
double pagex,pagey;	/* printable page-size   in mm	*/
double borderl,borderb;	/* page border in mm (left,bott)*/
double pswidth;		/* page size in psunits		*/
double pixelx;		/* pixel per page		*/
double posterx,postery;	/* poster size 	in mm		*/
int pagesx,pagesy;	/* nr. of pages			*/
int x,y;
int cx1,cy1,cx2,cy2;

char* usage = "-posterx|-postery <s> [-pagex <x>] [-pagey <y>] [-borderl <x>] [-borderb <y>] [-inches|-mm (default)] [-pnmonly] pnmfile\n";

int pm_keymatch(char* arg, char* opt, int minlen)
{
        for (; *arg && *arg == *opt; arg++, opt++)
                minlen--;
        return !*arg && minlen <= 0;
}

void pm_usage(char* u)
{
        fprintf(stderr, "usage: pnmposter %s\n", u);
        exit(1);
}

int main(int argc, char *argv[]) {

  int argn=1, ppmonlyflag, mmflag;
  char *index;

  /*---- DEFAULTS -------*/
  ppmonlyflag=0;
  mmflag=1; /* all in mm */
  dpi=300;

    /* Check for flags. */
    while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' )
	{
	if ( pm_keymatch( argv[argn], "-posterx", 8 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &posterx ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-postery", 8 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &postery ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-pagex", 6 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &pagex ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-pagey", 6 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &pagey ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-borderl", 8 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &borderl ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-borderb", 8 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%lf", &borderb ) != 1 )
		pm_usage( usage );
	    }
	else if ( pm_keymatch( argv[argn], "-mm", 3 ) )
	    mmflag = 1;
	else if ( pm_keymatch( argv[argn], "-inches", 5 ) )
	    mmflag = 0;
	else if ( pm_keymatch( argv[argn], "-pnmonly", 4 ) )
	    ppmonlyflag = 1;

	else if ( pm_keymatch( argv[argn], "-dpi", 2 ) )
	    {
	    ++argn;
	    if ( argn == argc || sscanf( argv[argn], "%d", &dpi ) != 1 )
		pm_usage( usage );
	    }
	else
	    pm_usage( usage );
	++argn;
	}

    if ( argn < argc )
	{
        strcpy(filename,argv[argn]);
        fp=fopen(filename,"r");
	++argn;
	}

    if ( argn != argc )
	pm_usage( usage );


  if (mmflag) { /* convert mm to inches */
    posterx*=INCH;
    postery*=INCH;
    pagex*=INCH;
    pagey*=INCH;
    borderl*=INCH;
    borderb*=INCH;
  }

  if (borderl==0) borderl=10 *INCH;
  if (borderb==0) borderb=10 *INCH;
  if (pagex==0) pagex=195 *INCH;
  if (pagey==0) pagey=280 *INCH;

  
  if (fp==0) {printf("error opening file %s.\n",filename);return 1;}
  fscanf(fp,"%s\n",&casper);
  if (casper[0]!='P' || casper[1]<'1' || casper[1]>'6') {printf("no ppm.\n");return 1;}
  
  do {
    fgets(casper,255, fp);
    /*printf("$%s",casper);*/
  } while (casper[0]=='#');

  sscanf(casper,"%i %i",&imagex,&imagey);

  /* make outputname like filename without .??? */
  index=(char*)strrchr(filename,'/');
  if (!index) index=filename; else index++;
  strcpy(outname,index);
  index=(char*)strrchr(outname,'.');
  if (!index) index=outname+strlen(outname);
  strcpy(index,"_");
 
  /* CALCULATE POSTERSIZE RELATIVE TO IMAGESIZE */
  if (postery==0.0) postery=posterx*imagey/imagex;
  else posterx=postery*imagex/imagey;
  if (posterx==0.0) {printf("You must specify posterx OR postery\n");return 1;}
  

  pixelx=imagex/posterx*pagex;

  pswidth = pagex * 72.0; /* inch in postscript units */

  devpix=dpi/72.0 + 0.5;
  scale=pswidth/pixelx*dpi/72.0/devpix;

  /* printf("pswidth: %f pixelx: %f scale: %f\n",pswidth,pixelx,scale); */

  pagesx=posterx/pagex; if (fmod(posterx,pagex)>0) pagesx+=1;
  pagesy=postery/pagey; if (fmod(postery,pagey)>0) pagesy+=1;

  printf("image size:  %i x %i pixel\n",imagex,imagey);  
  if (mmflag) {
	printf("poster size: %6.1f x %6.1f mm\n",posterx / INCH,postery / INCH);
	printf("page   size: %6.1f x %6.1f mm\n",pagex / INCH,pagey / INCH);
  }
  else {
	printf("poster size: %6.1f x %6.1f inch\n",posterx,postery);
	printf("page   size: %6.1f x %6.1f inch\n",pagex ,pagey );
  }
  printf("pages:       %i x %i\n",pagesx,pagesy);

  for (y=0; y<pagesy; y++) {
    for (x=0; x<pagesx; x++) {
      cx1=(double)x* pagex *((double)imagex/posterx);
      cx2=((double)(x+1)* pagex *((double)imagex/posterx));
      if (cx2>imagex) cx2=imagex;

      cy1=(double)y* pagey *((double)imagey/postery);
      cy2=((double)(y+1)* pagey *((double)imagey/postery));
      if (cy2>imagey) cy2=imagey;
      
      sprintf(tmpname,"%s%02i%02i",outname,y,x);
      printf("cut page y%i x%i (x:%i y:%i w:%i h:%i) to file %s\n",y,x,cx1,cy1,cx2-cx1,cy2-cy1,tmpname);
      sprintf(casper,"pnmcut %i %i %i %i %s > %s\n",cx1,cy1,
      	cx2-cx1,cy2-cy1, filename, tmpname);

      system(casper);
      sprintf(casper,"pnmtops -scale %f -width %f -height %f -noturn %s > %s.ps; rm %s\n",
	scale,(pagex+2*borderl),(pagey+2*borderb), tmpname, tmpname, tmpname);
      if (!ppmonlyflag) system(casper); 

    }
  }

  

}

